// Copyright 2025 Zentrum für Digitale Souveränität der Öffentlichen Verwaltung (ZenDiS) GmbH.
// SPDX-License-Identifier: MIT

import { ChevronDownIcon } from '@heroicons/react/24/outline'
import { SarifApiResponse } from '../lib/apiClient'
import { cn } from '../lib/utils'
import { Badge } from './ui/badge'

interface Props {
    currentScan: SarifApiResponse
}

const translateCriteriaStatus = (status: string) => {
    switch (status) {
        case 'open':
            return 'Offen'
        case 'pass':
            return 'Bestanden'
        case 'fail':
            return 'Nicht bestanden'
        default:
            return status
    }
}

const badgeTranslation: Record<string, Record<string, string>> = {
    maintained: {
        bronze: 'Aktive Wartung',
        silver: 'Verlässliche Wartung',
        gold: 'Krisensichere Wartung',
    },
    reused: {
        bronze: 'Bereit zur Nutzung',
        silver: 'Aktiv genutzt',
        gold: 'Häufig aktiv genutzt',
    },
    'open-source': {
        gold: 'Open Source lizenziert',
    },
}

const translateBadge = (badgeId: string, level: string): string => {
    if (badgeId in badgeTranslation) {
        if (level in badgeTranslation[badgeId]) {
            return badgeTranslation[badgeId][level]
        }
    }
    return badgeId + ' ' + level
}

export default function ResultTable({ currentScan }: Props) {
    return (
        <div className="">
            {currentScan?.runs[0].properties.badges
                .sort((a, b) => a.badgeId.localeCompare(b.badgeId))
                .map((badge) => (
                    <div
                        key={`${badge.badgeId}-${badge.badgeLevel}`}
                        className="mt-8 rounded-lg bg-gray-100 px-4 py-2"
                    >
                        <details className="">
                            <summary className="relative flex cursor-pointer flex-row justify-between font-medium text-gray-900 md:text-lg">
                                <span>
                                    {translateBadge(
                                        badge.badgeId,
                                        badge.badgeLevel,
                                    )}
                                    <Badge
                                        variant={
                                            badge.badgeGranted
                                                ? 'default'
                                                : 'destructive'
                                        }
                                        className="ml-2"
                                    >
                                        {badge.badgeGranted
                                            ? 'Verliehen'
                                            : 'Verweigert'}
                                    </Badge>
                                </span>
                                <div className="flex flex-row items-center gap-2">
                                    <span className="hidden text-sm text-gray-500 md:inline-block">
                                        Mehr informationen
                                    </span>
                                    <ChevronDownIcon className="w-4" />
                                </div>
                            </summary>
                            <div className="md:mt-8">
                                <div className="overflow-x-aut">
                                    <div className="inline-block min-w-full py-2 align-middle">
                                        <table className="flex min-w-full divide-gray-400 md:table md:divide-y">
                                            <thead className="hidden md:table-header-group">
                                                <tr className="grid grid-cols-6">
                                                    <th
                                                        scope="col"
                                                        className="col-span-1 py-3.5 pr-3 text-left font-semibold lg:pl-8"
                                                    >
                                                        Überprüfung
                                                    </th>
                                                    <th
                                                        scope="col"
                                                        className="col-span-1 px-3 py-3.5 text-left font-semibold"
                                                    >
                                                        Status
                                                    </th>
                                                    <th
                                                        scope="col"
                                                        className="col-span-2 px-3 py-3.5 text-left font-semibold"
                                                    >
                                                        Beschreibung
                                                    </th>
                                                    <th
                                                        scope="col"
                                                        className="col-span-2 px-3 py-3.5 text-left font-semibold"
                                                    >
                                                        Nachweis
                                                    </th>
                                                </tr>
                                            </thead>
                                            <tbody className="divide-y divide-gray-300">
                                                {badge.badgeExplanation.criteria.map(
                                                    (criteria) => (
                                                        <tr
                                                            key={`${badge.badgeId}-${badge.badgeLevel}-${criteria.ruleId}`}
                                                            className="grid grid-cols-1 md:grid-cols-6"
                                                        >
                                                            <td
                                                                title={
                                                                    criteria.ruleId
                                                                }
                                                                className="col-span-1 overflow-hidden overflow-ellipsis whitespace-nowrap py-4 font-medium text-gray-900 md:pr-3 lg:pl-8"
                                                            >
                                                                {
                                                                    criteria.ruleId.split(
                                                                        ':',
                                                                    )[0]
                                                                }
                                                            </td>
                                                            <td className="col-span-1 whitespace-nowrap py-4 md:px-3">
                                                                <div className="flex items-center justify-end gap-x-2 md:justify-start">
                                                                    <span className="relative flex size-2">
                                                                        <span
                                                                            className={cn(
                                                                                'absolute inline-flex h-full w-full animate-ping rounded-full opacity-75',
                                                                                criteria.status ===
                                                                                    'pass'
                                                                                    ? 'bg-green-400'
                                                                                    : criteria.status ===
                                                                                        'open'
                                                                                      ? 'bg-gray-400'
                                                                                      : 'bg-red-400',
                                                                            )}
                                                                        ></span>
                                                                        <span
                                                                            className={cn(
                                                                                'relative inline-flex size-2 rounded-full',
                                                                                criteria.status ===
                                                                                    'pass'
                                                                                    ? 'bg-green-400'
                                                                                    : criteria.status ===
                                                                                        'open'
                                                                                      ? 'bg-gray-400'
                                                                                      : 'bg-red-400',
                                                                            )}
                                                                        ></span>
                                                                    </span>
                                                                    <span className="w-full">
                                                                        {translateCriteriaStatus(
                                                                            criteria.status,
                                                                        )}
                                                                    </span>
                                                                </div>
                                                            </td>
                                                            <td className="col-span-2 py-4 md:px-3">
                                                                <b className="md:hidden">
                                                                    Beschreibung:{' '}
                                                                </b>
                                                                {
                                                                    criteria.description
                                                                }
                                                            </td>
                                                            <td className="col-span-2 py-4 md:px-3">
                                                                <b className="md:hidden">
                                                                    Nachweis:{' '}
                                                                </b>
                                                                {
                                                                    criteria.evidence
                                                                }
                                                            </td>
                                                        </tr>
                                                    ),
                                                )}
                                            </tbody>
                                        </table>
                                    </div>
                                </div>
                            </div>
                        </details>
                    </div>
                ))}
        </div>
    )
}
